"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const three_1 = require("three");
function FocuserKeyboard(Base) {
    class FocuserKeyboardMixIn extends Base {
        defaults() {
            return Object.assign(Object.assign({}, super.defaults()), { focusRaycaster: new three_1.Raycaster(), focusMousePos: new three_1.Vector2() });
        }
        update(p) {
            const viewer = this.project.viewer;
            const plyr = this.project.player;
            if (!viewer)
                return;
            const { w: width, h: height } = viewer.dims;
            const offset = this.mousePos.offset;
            this.focusMousePos.x = (offset.x / width) * 2 - 1;
            this.focusMousePos.y = 0 - ((offset.y / height) * 2 - 1);
            this.focusRaycaster.setFromCamera(this.focusMousePos, plyr.camera);
            const targets = this.focusTargets.map(m => m);
            const intersects = this.focusRaycaster.intersectObjects(targets);
            const hitTargetIdx = intersects.length > 0 ? targets.indexOf(intersects[0].object) : -1;
            if (hitTargetIdx >= 0)
                this.focusTargets[hitTargetIdx].focus();
            this.focusTargets.forEach((target, idx) => {
                if (idx !== hitTargetIdx && target.isFocused)
                    target.blur();
            });
            super.update(p);
        }
    }
    return FocuserKeyboardMixIn;
}
exports.default = FocuserKeyboard;
//# sourceMappingURL=FocuserKeyboard.js.map