"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const ControllerType_1 = __importDefault(require("../../enums/ControllerType"));
const Selector_1 = __importDefault(require("../selector/Selector"));
const GrabberKeyboard_1 = __importDefault(require("./GrabberKeyboard"));
const GrabberQuest2_1 = __importDefault(require("./GrabberQuest2"));
function Grabber() {
    return [...(0, Selector_1.default)(), GrabberMixin];
}
exports.default = Grabber;
function GrabberMixin(Base) {
    let CoreBase = GrabberCore(Base);
    switch (Base.controllerType) {
        case ControllerType_1.default.KEYBOARD:
            return (0, GrabberKeyboard_1.default)(CoreBase);
        case ControllerType_1.default.QUEST2:
            return (0, GrabberQuest2_1.default)(CoreBase);
    }
    return CoreBase;
}
function GrabberCore(Base) {
    class GrabberCoreMixIn extends Base {
        defaults() {
            return Object.assign(Object.assign({}, super.defaults()), { grabbed: [] });
        }
        drag(v) {
        }
        grab(v) {
            v.grab();
            if (this.grabbed.indexOf(v) >= 0)
                return;
            this.grabbed.push(v);
            this.onChangeGrabbed(this.grabbed);
        }
        ungrab(v) {
            v.ungrab();
            if (this.grabbed.indexOf(v) < 0)
                return;
            this.grabbed = this.grabbed.filter(vv => vv !== v);
            this.onChangeGrabbed(this.grabbed);
        }
        ungrabAll() {
            this.grabbed.forEach(v => v.ungrab());
            this.grabbed = [];
            this.onChangeGrabbed(this.grabbed);
        }
        onChangeGrabbed(v) { }
    }
    return GrabberCoreMixIn;
}
//# sourceMappingURL=Grabber.js.map