"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const three_1 = require("three");
const ControllerType_1 = __importDefault(require("../../enums/ControllerType"));
const WalkerKeyboard_1 = __importDefault(require("./WalkerKeyboard"));
const WalkerQuest2_1 = __importDefault(require("./WalkerQuest2"));
function Walker(fly = false) {
    return [
        function (Base) {
            const CoreBase = WalkerCore(fly, Base);
            switch (Base.controllerType) {
                case ControllerType_1.default.KEYBOARD:
                    return (0, WalkerKeyboard_1.default)(CoreBase);
                case ControllerType_1.default.QUEST2:
                    return (0, WalkerQuest2_1.default)(CoreBase);
            }
            return CoreBase;
        },
    ];
}
exports.default = Walker;
function WalkerCore(fly, Base) {
    const maxTurn = 45;
    class WalkerCoreMixIn extends Base {
        defaults() {
            return Object.assign(Object.assign({}, super.defaults()), { impulsion: new three_1.Vector3(), rotation: new three_1.Vector3(), walkSpeed: 8, strafeSpeed: 4, lookUpSpeed: Math.PI / 2, yawSpeed: Math.PI / 2, pitchSpeed: Math.PI / 2, rollSpeed: Math.PI / 2 });
        }
        update(p) {
            const plyr = this.project.player;
            if (p === null || p === void 0 ? void 0 : p.delta) {
                if (this.impulsion.z)
                    plyr.walkBy(this.impulsion.z * this.walkSpeed * p.delta);
                if (this.impulsion.x)
                    plyr.strafeBy(this.impulsion.x * this.strafeSpeed * p.delta);
                if (this.rotation.y)
                    plyr.yawBy(this.rotation.y * this.yawSpeed * p.delta);
                if (fly) {
                    if (this.rotation.x)
                        plyr.pitchBy(this.rotation.x * this.pitchSpeed * p.delta);
                    if (this.rotation.z)
                        plyr.rollBy(this.rotation.z * this.rollSpeed * p.delta);
                }
                else {
                    if (this.rotation.x)
                        plyr.lookUpBy(this.rotation.x * this.lookUpSpeed * p.delta);
                }
            }
            super.update(p);
        }
    }
    return WalkerCoreMixIn;
}
//# sourceMappingURL=Walker.js.map