"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const XRController_1 = __importDefault(require("../core/XRController"));
const ControllerType_1 = __importDefault(require("../enums/ControllerType"));
class Quest2Controller extends XRController_1.default {
    defaults() {
        return Object.assign(Object.assign({}, super.defaults()), { controllerType: ControllerType_1.default.QUEST2, speed: 0.5 });
    }
    init() {
        super.init();
        this._onInputSourcesChange = this.onInputSourcesChange.bind(this);
        const s = this.project.session;
        s.addEventListener('inputsourceschange', this._onInputSourcesChange);
        s.requestReferenceSpace('viewer').then(a => {
            this.refSpace = a;
        });
    }
    update(p) {
        var _a, _b, _c, _d;
        super.update(p);
        const plyr = this.project.player;
        if (plyr.leftHand && ((_a = this.left) === null || _a === void 0 ? void 0 : _a.targetRaySpace) && this.refSpace && !!p.frame)
            plyr.leftHand.pose = p.frame.getPose((_b = this.left) === null || _b === void 0 ? void 0 : _b.targetRaySpace, this.refSpace);
        if (plyr.rightHand && ((_c = this.right) === null || _c === void 0 ? void 0 : _c.targetRaySpace) && this.refSpace && !!p.frame)
            plyr.rightHand.pose = p.frame.getPose((_d = this.right) === null || _d === void 0 ? void 0 : _d.targetRaySpace, this.refSpace);
    }
    onInputSourcesChange() {
        var _a;
        const s = this.project.session;
        const p = this.project.player;
        this.left = undefined;
        this.right = undefined;
        (_a = s === null || s === void 0 ? void 0 : s.inputSources) === null || _a === void 0 ? void 0 : _a.forEach(i => {
            if (i.handedness === 'left')
                this.left = i;
            if (i.handedness === 'right')
                this.right = i;
        });
        if (this.left)
            p.leftHand.visible = true;
        if (this.right)
            p.rightHand.visible = true;
    }
    destroy() {
        const s = this.project.session;
        const p = this.project.player;
        s.removeEventListener('inputsourceschange', this._onInputSourcesChange);
        p.leftHand.visible = false;
        p.rightHand.visible = false;
    }
}
exports.default = Quest2Controller;
Quest2Controller.controllerType = ControllerType_1.default.QUEST2;
//# sourceMappingURL=Quest2Controller.js.map