"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.XRPlayer = void 0;
const uniq_1 = __importDefault(require("lodash/uniq"));
const three_1 = require("three");
const KeyboardController_1 = __importDefault(require("../controllers/KeyboardController"));
const Quest2Controller_1 = __importDefault(require("../controllers/Quest2Controller"));
const RenderMode_1 = __importDefault(require("../enums/RenderMode"));
const PlayerConsole_1 = __importDefault(require("../playertools/PlayerConsole"));
const PlayerHand_1 = __importDefault(require("../playertools/PlayerHand"));
const XRObject_1 = __importDefault(require("./XRObject"));
class XRPlayer extends XRObject_1.default {
    defaults() {
        return Object.assign(Object.assign({}, super.defaults()), { xr: false });
    }
    init() {
        var _a, _b;
        super.init();
        this.setController(KeyboardController_1.default);
        this.camera = new three_1.PerspectiveCamera(50, 1, 0.01, 1000);
        this.add(this.camera);
        this.leftHand = new PlayerHand_1.default();
        this.camera.add(this.leftHand);
        this.rightHand = new PlayerHand_1.default();
        this.camera.add(this.rightHand);
        this.debugConsole = new PlayerConsole_1.default();
        this.debugConsole.position.set(0, 0, -2);
        this.debugConsole.log('Hello world');
        this.camera.add(this.debugConsole);
        this.setSize(((_a = this.project.viewer) === null || _a === void 0 ? void 0 : _a.dims.w) || 100, ((_b = this.project.viewer) === null || _b === void 0 ? void 0 : _b.dims.h) || 100);
    }
    update(o) {
        var _a;
        (_a = this.controller) === null || _a === void 0 ? void 0 : _a.update(o);
    }
    setController(C) {
        var _a, _b;
        (_a = this.controller) === null || _a === void 0 ? void 0 : _a.destroy();
        (_b = (0, uniq_1.default)(this.project.scene.controllerBehaviours)) === null || _b === void 0 ? void 0 : _b.forEach(B => (C = B(C)));
        this.controller = new C({ project: this.project });
    }
    setRenderMode(v) {
        this.xr = v === RenderMode_1.default.XR;
        if (this.camera)
            this.camera.position.y = this.xr ? 0 : 1;
        this.setController(this.xr ? Quest2Controller_1.default : KeyboardController_1.default);
    }
    setSize(width, height) {
        this.camera.aspect = width / height;
        this.camera.updateProjectionMatrix();
    }
    walkBy(v) {
        if (v)
            this.translateZ(v);
    }
    strafeBy(v) {
        if (v)
            this.translateX(v);
    }
    lookUpBy(v) {
        if (v)
            this.camera.rotateX(v);
    }
    yawBy(v) {
        if (v)
            this.rotateY(v);
    }
    pitchBy(v) {
        if (v)
            this.rotateX(v);
    }
    rollBy(v) {
        if (v)
            this.rotateZ(v);
    }
    destroy() {
        var _a, _b, _c, _d;
        (_a = this.controller) === null || _a === void 0 ? void 0 : _a.destroy();
        (_b = this.leftHand) === null || _b === void 0 ? void 0 : _b.destroy();
        (_c = this.rightHand) === null || _c === void 0 ? void 0 : _c.destroy();
        (_d = this.debugConsole) === null || _d === void 0 ? void 0 : _d.destroy();
    }
}
exports.XRPlayer = XRPlayer;
exports.default = XRPlayer;
//# sourceMappingURL=XRPlayer.js.map