"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.createTriMeshShape = void 0;
const three_1 = require("three");
const ShapeFit_1 = __importDefault(require("../../enums/ShapeFit"));
const ShapeType_1 = __importDefault(require("../../enums/ShapeType"));
const computeScale_1 = __importDefault(require("../computeScale"));
const finishCollisionShape_1 = __importDefault(require("../finishCollisionShape"));
const setDefaultOptions_1 = __importDefault(require("./setDefaultOptions"));
exports.createTriMeshShape = (function () {
    const va = new three_1.Vector3();
    const vb = new three_1.Vector3();
    const vc = new three_1.Vector3();
    const matrix = new three_1.Matrix4();
    return function (vertices, matrices, indexes, matrixWorld, options = {}) {
        options.type = ShapeType_1.default.MESH;
        (0, setDefaultOptions_1.default)(options);
        if (options.fit === ShapeFit_1.default.MANUAL) {
            console.warn('cannot use fit: manual with type: mesh');
            return null;
        }
        const scale = (0, computeScale_1.default)(matrixWorld, options);
        const bta = new Ammo.btVector3();
        const btb = new Ammo.btVector3();
        const btc = new Ammo.btVector3();
        const triMesh = new Ammo.btTriangleMesh(true, false);
        for (let i = 0; i < vertices.length; i++) {
            const components = vertices[i];
            const index = indexes[i] ? indexes[i] : null;
            matrix.fromArray(matrices[i]);
            if (index) {
                for (let j = 0; j < index.length; j += 3) {
                    const ai = index[j] * 3;
                    const bi = index[j + 1] * 3;
                    const ci = index[j + 2] * 3;
                    va.set(components[ai], components[ai + 1], components[ai + 2]).applyMatrix4(matrix);
                    vb.set(components[bi], components[bi + 1], components[bi + 2]).applyMatrix4(matrix);
                    vc.set(components[ci], components[ci + 1], components[ci + 2]).applyMatrix4(matrix);
                    bta.setValue(va.x, va.y, va.z);
                    btb.setValue(vb.x, vb.y, vb.z);
                    btc.setValue(vc.x, vc.y, vc.z);
                    triMesh.addTriangle(bta, btb, btc, false);
                }
            }
            else {
                for (let j = 0; j < components.length; j += 9) {
                    va.set(components[j + 0], components[j + 1], components[j + 2]).applyMatrix4(matrix);
                    vb.set(components[j + 3], components[j + 4], components[j + 5]).applyMatrix4(matrix);
                    vc.set(components[j + 6], components[j + 7], components[j + 8]).applyMatrix4(matrix);
                    bta.setValue(va.x, va.y, va.z);
                    btb.setValue(vb.x, vb.y, vb.z);
                    btc.setValue(vc.x, vc.y, vc.z);
                    triMesh.addTriangle(bta, btb, btc, false);
                }
            }
        }
        const localScale = new Ammo.btVector3(scale.x, scale.y, scale.z);
        triMesh.setScaling(localScale);
        Ammo.destroy(localScale);
        let collisionShape;
        if (options.concave)
            collisionShape = new Ammo.btBvhTriangleMeshShape(triMesh, true, true);
        else
            collisionShape = new Ammo.btConvexTriangleMeshShape(triMesh, true);
        collisionShape.resources = [triMesh];
        Ammo.destroy(bta);
        Ammo.destroy(btb);
        Ammo.destroy(btc);
        (0, finishCollisionShape_1.default)(collisionShape, options);
        return collisionShape;
    };
})();
exports.default = exports.createTriMeshShape;
//# sourceMappingURL=createTriMeshShape.js.map